import React from 'react';
import isFunction from 'lodash.isfunction';

export default class HorizontalAxis extends React.Component {
  static propTypes = {
    height: React.PropTypes.number.isRequired,
    left: React.PropTypes.number,
    top: React.PropTypes.number,
    width: React.PropTypes.number.isRequired
  };
  static defaultProps = {
    left: 0,
    top: 0,
    tickSize: 20,
    numTicks: 11,
    fontSize: 20,
    tickStyle: {stroke: 'black', strokeWidth: 2},
    gridLineStyle: {stroke: '#aaa', strokeWidth: 2},
    width: 0
  };

  makeGridLines () {
    const {height, gridLineStyle, scale, numTicks, x, y} = this.props;
    const tickValues = scale.ticks(numTicks);

    let tickGridLines = [];
    if (this.props.gridLineStyle) {
      tickGridLines = tickValues.map((tickValue, idx) => {
        const xPos = x + scale(tickValue);
        return (
          <g className={"vertical gridline"}
              key={idx}
              y={y}
          >
            <line
                style={gridLineStyle}
                x1={xPos}
                x2={xPos}
                y1={0}
                y2={y + height}
            />
          </g>
        );
      });
    }
    return tickGridLines;
  }

  render () {
    const {fontSize, scale, x, y, numTicks, height} = this.props;
    const tickValues = scale.ticks(numTicks);
    const tickY = y + height;
    const ticks = tickValues.map((tickValue, idx, collection) => {
      const xPos = x + scale(tickValue);
      const tickSize = isFunction(this.props.tickSize) ? this.props.tickSize(tickValue, idx, collection) : this.props.tickSize;
      return (
        <g className={"tick"}
            key={idx}
            y={y}
        >
            <line
                style={this.props.tickStyle}
                x1={xPos}
                x2={xPos}
                y1={tickY}
                y2={tickY + tickSize}
            />
            <text
                dx={'0'}
                dy={tickSize + fontSize}
                fontSize={fontSize}
                style={{}}
                textAnchor={"middle"}
                x={xPos}
                y={tickY}
            >{tickValue}</text>
        </g>
      );
    });
    const gridLines = this.makeGridLines();
    return (
        <g
            className={"axis horizontal"}
            transform={`translate(${this.props.left}, ${this.props.top})`}
        >
            <g className={"ticks"}>{ticks}</g>
            <g className={"grid horizontal"}>{gridLines}</g>
        <text
            className="label"
            style={{textAnchor: 'middle', fontSize: '0.25em'}}
            x={this.props.width / 2}
            y="-6"
        >{this.props.label}</text>
        <line
            style={{stroke: '#e0e0e0', strokeWidth: 2}}
            x1={x + scale.range()[0]}
            x2={x + scale.range()[1]}
            y1={tickY}
            y2={tickY}
        />
      </g>
    );
  }
}
