import React from 'react';
import isFunction from 'lodash.isfunction';

export default class VerticalAxis extends React.Component {
  static propTypes = {
    data: React.PropTypes.array.isRequired,
    height: React.PropTypes.number.isRequired,
    scale: React.PropTypes.func.isRequired,
    width: React.PropTypes.number.isRequired
  };

  static defaultProps = {
    fontSize: '1.5em',
    gridLineStyle: {stroke: '#aaa', strokeWidth: 2},
    tickSize: 50,
    tickStyle: {stroke: 'black', strokeWidth: 2},
    numTicks: 10
  };

  makeGridLines () {
    const {chartWidth, height, width, gridLineStyle, scale, numTicks, x, y} = this.props;
    const tickValues = scale.ticks(numTicks);
    let tickGridLines = [];
    if (this.props.gridLineStyle) {
      tickGridLines = tickValues.map((tickValue, idx) => {
        const yPos = height - scale(tickValue) - y;

        return (
          <g className={"horizontal gridline"}
              key={idx}
              y={y}
          >
              <line
                  style={gridLineStyle}
                  x1={x + width}
                  x2={x + width + chartWidth}
                  y1={yPos}
                  y2={yPos}
              />
          </g>
        );
      });
    }
    return tickGridLines;
  }
  render () {
    const {fontSize, tickStyle, scale, x, y, numTicks, height} = this.props;
    const tickValues = scale.ticks(numTicks);
    const gridLines = this.makeGridLines();
    const ticks = tickValues.map((tickValue, idx, collection) => {
      const yPos = height - scale(tickValue) - y;
      const tickSize = isFunction(this.props.tickSize) ?
          this.props.tickSize(tickValue, idx, collection) : this.props.tickSize;
      return (
        <g className={"tick"}
            key={idx}
            y={yPos}
        >
          <line
              style={tickStyle}
              x1={x}
              x2={x + tickSize}
              y1={yPos}
              y2={yPos}
          />
          <text
              color={'white'}
              dx={'0'}
              fill={"black"}
              fontSize={fontSize}
              stroke={"black"}
              style={{}}
              x={x}
              y={yPos}
          >{tickValue}</text>

        </g>
      );
    });
    return (
      <g
          className={"axis vertical"}
          key={this.props.id}
      >{ticks}
<g className={"grid horizontal"}>{gridLines}</g>
      </g>

    );
  }
}
