import React from 'react';
import HorizontalAxis from '../../components/HorizontalAxis';
import VerticalAxis from '../../components/VerticalAxis';

const horizontalAxis = HorizontalAxis;
const verticalAxis = VerticalAxis;

const AxisChart = Component => class extends React.Component {

  static defaultProps = {
    axisLeftWidth: 100,
    axisLeftComponent: verticalAxis,
    axisLeftScale: null,
    axisBottomComponent: horizontalAxis,
    axisBottomHeight: 100,
    axisBottomScale: null,
    axisRightComponent: null,
    axisRightScale: null,
    axisRightWidth: 0,
    axisTopComponent: null,
    axisTopHeight: 0,
    axisTopScale: null,
    x: 0,
    y: 0
  };
  static displayName = 'AxisDataSeries[' + (Component.displayName || Component.name) + ']';
  render () {
    const {axisTopComponent, axisRightComponent, axisBottomComponent, axisLeftComponent, axisLeftWidth, axisTopHeight,
      width, height, x, y} = this.props;
    const axisLeftPadding = (axisLeftComponent && axisLeftWidth) || 0;
    const chartWidth = width - (axisLeftPadding + (axisRightComponent && this.props.axisRightWidth) || 0);
    const chartHeight = height - (((axisTopComponent && this.props.axisTopHeight) || 0)
      + ((axisBottomComponent && this.props.axisBottomHeight) || 0));
    const hScale = this.props.hScale.range([x + axisLeftPadding, x + axisLeftPadding + chartWidth]);
    const vScale = this.props.vScale.range([y, (y + chartHeight)]);
    if (axisBottomComponent) {
      axisBottomComponent.y = chartHeight;
    }
    if (axisLeftComponent) {
      axisLeftComponent.y = axisBottomComponent.height;
      axisLeftComponent.scale = vScale;
    }
    return (
      <g className={"axis-chart"}>

        {
          [{component: axisTopComponent, scale: this.props.axisTopScale || hScale, gridLineStyle: null},
          {component: axisRightComponent, scale: this.props.axisRightScale || vScale, gridLineStyle: null},
          {component: axisBottomComponent, scale: this.props.axisBottomScale || hScale, gridLineStyle: {stroke: '#e0e0e0', strokeWidth: 2}},
          {component: axisLeftComponent, scale: this.props.axisLeftScale || vScale, gridLineStyle: {stroke: '#e0e0e0', strokeWidth: 2}, width: axisLeftWidth}
          ].map((axis, idx) => {
            const key = `axis-${idx}`;

            if (axis.component) {
              const AxisComponent = axis.component;
              return (
                <AxisComponent
                    {...this.props}
                    {...axis}
                    chartWidth={chartWidth}
                    displayName={AxisComponent.displayName}
                    gridLineStyle={axis.gridLineStyle}
                    hScale={hScale}
                    height={chartHeight}
                    id={key}
                    key={key}
                    scale={axis.scale}
                    vScale={vScale}
                    width={axis.width}
                />
              );
            }
          })}
        <Component
            {...this.props}
            displayName={Component.displayName}
            hScale={hScale}
            height={chartHeight}
            vScale={vScale}
            width={chartWidth}
            x={x + (axisLeftComponent && axisLeftWidth) || 0}
            y={y + (axisTopComponent && axisTopHeight) || 0}
        />
      </g>
    );
  }
};

export default AxisChart;
