import React from 'react';
import find from 'lodash.find';

const SelectableWrapper = Component => class extends React.Component {
  constructor (props) {
    super(props);
  }
  static displayName = `SelectableWrapper[${Component.displayName || Component.name}]`;
  static propTypes = {
    id: React.PropTypes.string.isRequired,
    onChartSelectEnd: React.PropTypes.func,
    onChartSelectMove: React.PropTypes.func,
    onChartSelectStart: React.PropTypes.func,
    onDataSelection: React.PropTypes.func
  };

  static defaultProps = {
    onChartSelectEnd: () => {},
    onChartSelectMove: () => {},
    onChartSelectStart: () => {}
  };

  getDataCoords (svgCoords) {
    const {hScale, vScale} = this.props;
    const vRange = vScale.range();
    const vScaleInvertedRange = vScale.copy().range([vRange[1], vRange[0]]);
    return {x: hScale.invert(svgCoords.x), y: vScaleInvertedRange.invert(svgCoords.y)};
  }

  getSelectionRect () {
    const {dataSeriesSelections, id, hScale} = this.props;
    const selectionCoords = find(dataSeriesSelections, (dataSeriesSelection) => {return dataSeriesSelection.id === id;});
    const {xMin, xMax} = selectionCoords;
    const x = hScale(xMin);
    const width = hScale(xMax) - x;
    const y = this.props.y;
    const height = this.props.height;
    return (
      <rect
          fill="#FF0000"
          fillOpacity={0.1}
          height={height}
          pointerEvents={"none"}
          width={width}
          x={x}
          y={y}
      />);
  }

  render () {
    const {id, x, y, width, height} = this.props;
    return (
      <g
          onMouseDown={(ev) => {
            const dataCoords = this.getDataCoords({x: ev.nativeEvent.offsetX, y: ev.nativeEvent.offsetY - y});
            this.props.onChartSelectStart(id, dataCoords);
          }}
          onMouseMove={(ev) => {
            const dataCoords = this.getDataCoords({x: ev.nativeEvent.offsetX, y: ev.nativeEvent.offsetY - y});
            this.props.onChartSelectMove(id, dataCoords);
          }}
          onMouseUp={(ev) => {
            const {dataSeriesSelections} = this.props;
            const selectionCoords = find(dataSeriesSelections, (dataSeriesSelection) => {return dataSeriesSelection.id === id;});
            const dataCoords = this.getDataCoords({x: ev.nativeEvent.offsetX, y: ev.nativeEvent.offsetY - y});
            this.props.onChartSelectEnd(id, dataCoords);
            this.props.onDataSelection(id, selectionCoords);
          }}
          pointerEvents={"all"}

      >
        <rect
            pointerEvents={"none"}
            x1={x}
            x2={x + width}
            y1={height}
            y2={y + height}
        />

        <Component {...this.props} />
        {this.getSelectionRect()}
      </g>
    );
  }
};

export default SelectableWrapper;
