import React from 'react';
import find from 'lodash.find';
import {min, max} from 'd3-array';
import SelectableWrapper from './SelectableDataSeriesWrapper';

const ChartSelectionMultiplexer = Component => class extends React.Component {
  constructor (props) {
    super(props);
    this.state = {
      cursorLocation: {x: null, y: null},
      dataSeriesCoordinates: props.data.map((item, idx) => {
        return {
          dragging: false,
          id: item.id || idx,
          xMin: 0,
          xMax: 10,
          yMin: 0,
          yMax: 5,
          xStart: null,
          xEnd: null,
          yStart: null,
          yEnd: null
        };
      })
    };
    this.startChartSelection = this.startChartSelection.bind(this);
    this.moveChartSelection = this.moveChartSelection.bind(this);
    this.endChartSelection = this.endChartSelection.bind(this);
  }

  static defaultProps = {
    onDataSelection: () => {}
  };

  static propTypes = {
    onDataSelection: React.PropTypes.func.isRequired
  };

  startChartSelection (id, data) {
    const {dataSeriesCoordinates} = this.state;
    const {x, y} = data;
    dataSeriesCoordinates.map((item) => {
      item.dragging = (id === item.id);
      if (id === item.id) {
        // find(dataSeriesSelections, (dataSeriesSelection) => {return dataSeriesSelection.id === id;});
        item.xMin = x;
        item.xMax = x;
        item.xStart = x;
        item.xEnd = x;
        item.yStart = y;
        item.yEnd = y;
        item.yMin = y;
        item.yMax = y;
        item.dragging = true;
      }
      return item;
    });
    // eslint-disable-next-line react/no-set-state
    this.setState({dataSeriesCoordinates: dataSeriesCoordinates});
  }
  moveChartSelection (id, data) {
    const {dataSeriesCoordinates} = this.state;
    const selection = find(dataSeriesCoordinates, dataSeriesSelection => {return dataSeriesSelection.id === id;});
    if (selection.dragging) {
      selection.xEnd = data.x;
      selection.yEnd = data.y;
      const xMin = min([selection.xStart, selection.xEnd]);
      const xMax = max([selection.xStart, selection.xEnd]);
      const yMin = min([selection.yStart, selection.yEnd]);
      const yMax = max([selection.yStart, selection.yEnd]);
      // eslint-disable-next-line react/no-set-state
      this.setState({
        dataSeriesCoordinates: dataSeriesCoordinates.map(item => {
          if (item.id === id) {
            item.xMin = xMin;
            item.xMax = xMax;
            item.yMin = yMin;
            item.yMax = yMax;
          }
          return item;
        })
      });
    }
  }
  endChartSelection (id) {
    const {dataSeriesCoordinates} = this.state;
    // eslint-disable-next-line react/no-set-state
    this.setState({dataSeriesCoordinates: dataSeriesCoordinates.map(item => {
      if (item.id === id) {
        item.dragging = false;
      }
      return item;
    })});
  }

  render () {
    const WrappedComponentType = SelectableWrapper(this.props.dataSeriesComponent);
//    const OverlaidDataSeries = CrosshairOverlayWrapper(this.props.dataSeries);
    const dataSelectionCoordinates = this.state.dataSeriesCoordinates;
    return (
      <Component
          {...this.props}
          dataSeries={this.props.dataSeries}
          dataSeriesComponent={WrappedComponentType}
          dataSeriesSelections={dataSelectionCoordinates}
          onChartSelectEnd={this.endChartSelection}
          onChartSelectMove={this.moveChartSelection}
          onChartSelectStart={this.startChartSelection}
          onDataSelection={this.props.onDataSelection}
      />
    );
  }
};

export default ChartSelectionMultiplexer;
