import React from 'react';
import HorizontalCrosshairOverlay from '../../components/HorizontalCrosshairOverlay';

const SelectableChart = Component => class extends React.Component {
  constructor (props) {
    super(props);
    this.state = {
      startSelectionCoordinates: null,
      currentSelectionCoordinates: null,
      cursorLocation: {x: null, y: null},
      dragging: false
    };
    this.startDataSelect = this.startDataSelect.bind(this);
  }
  static displayName = 'SelectableDataSeries[' + (Component.displayName || Component.name) + ']';
  static propTypes = {
    onDragStart: React.PropTypes.func.isRequired,
    onSelectData: React.PropTypes.func.isRequired,
    selectionOverlayComponent: React.PropTypes.element.isRequired,
    x: React.PropTypes.number.isRequired,
    y: React.PropTypes.number.isRequired
  };

  static defaultProps = {
    x: 0,
    y: 0,
    selectionOverlayComponent: HorizontalCrosshairOverlay,
    onSelectData: () => {},
    onDragStart: () => {}
  };

  getDataCoords (svgCoords) {
    const {hScale, vScale} = this.props;
    return {x: hScale.invert(svgCoords.x), y: vScale.invert(svgCoords.y)};
  }

  startDataSelect (event) {
    event.stopPropagation();
    event.preventDefault();
    const x = event.nativeEvent.offsetX;
    const y = event.nativeEvent.offsetY;
    this.setState({ // eslint-disable-line react/no-set-state
      startSelectionCoordinates: {x, y},
      currentSelectionCoordinates: {x, y},
      dragging: true
    });
    this.props.onDragStart();
  }

  moveCursor (event) {
    event.stopPropagation();
    event.preventDefault();
    const x = event.nativeEvent.offsetX;
    const y = event.nativeEvent.offsetY;
    const mouseCoords = {x, y};
    // eslint-disable-next-line react/no-set-state
    this.setState({cursorLocation: mouseCoords});
    if (this.state.dragging) {
      // eslint-disable-next-line react/no-set-state
      this.setState({currentSelectionCoordinates: mouseCoords});
    }
  }

  endDataSelect (event) {
    event.stopPropagation();
    event.preventDefault();
    const x = event.nativeEvent.offsetX;
    const y = event.nativeEvent.offsetY;
    const startDataCoords = this.getDataCoords(this.state.startSelectionCoordinates);
    const dataCoords = this.getDataCoords({x, y});
    this.props.onSelectData({end: dataCoords, start: startDataCoords});
    this.setState({dragging: false});   // eslint-disable-line react/no-set-state
  }

  getSelectionDimensions () {
    if (this.state.startSelectionCoordinates && this.state.currentSelectionCoordinates) {
      const x = Math.min(this.state.startSelectionCoordinates.x, this.state.currentSelectionCoordinates.x);
      const y = this.props.y;
      const height = this.props.height;
      const width = Math.abs(this.state.startSelectionCoordinates.x - this.state.currentSelectionCoordinates.x);
      const rectStyle = {width: width, height: height, x: x, y: y, fill: '#768019', stroke: '#969974', strokeWidth: '0', opacity: '0.6'};
      const dragRectStyle = {width: width, height: height, x: x, y: y, fill: '#AAA', stroke: '#333', strokeWidth: '0.1', opacity: '0.6'};
      return this.state.dragging ? dragRectStyle : rectStyle;
    } else {
      return null;
    }
  }

  render () {
    const {x, y} = this.state.cursorLocation;
    const selectionDimensions = this.getSelectionDimensions();
    const OverlayComponent = this.props.selectionOverlayComponent || null;
    return (
      <g
          className={"selectable-chart"}
          onMouseDown={(ev) => this.startDataSelect(ev)}
          onMouseMove={(ev) => this.moveCursor(ev)}
          onMouseUp={(ev) => this.endDataSelect(ev)}
      >
          <Component
              {...this.props}
          />
          <rect
              className={"selection"}
              fill={"#FFA"}
              fillOpacity={0.2}
              {...selectionDimensions}
          />
          <OverlayComponent
              targetCoordX={x}
              targetCoordY={y}
              {...this.props}
          />
      </g>
    );
  }
};

export default SelectableChart;
